@@ -23,7 +23,7 @@
 
 #include <config.h>
 
-#ifndef HAVE_LIB_GLIB		/* Use this iff we're not using the 
+#ifndef HAVE_LIB_GLIB		/* Use this iff we're not using the
 				   `proper' glib */
 
 #include <stdlib.h>
@@ -43,8 +43,23 @@ static int FREE_LIST_ALLOCATE_CHUNK = 1024;
 GSList * free_list = NULL;
 GSList * allocated_list = NULL;
 
+#ifdef HAVE_STDATOMIC_H
+
+static atomic_char atomic_locker = ATOMIC_VAR_INIT(0);
+
+#define lock() atomic_lock( &atomic_locker )
+#define unlock() atomic_unlock( &atomic_locker )
+
+#else				/* #ifdef HAVE_STDATOMIC_H */
+
+#define lock()
+#define unlock()
+
+#endif				/* #ifdef HAVE_STDATOMIC_H */
+
 static
 void    allocate_free   ( void ) {
+    lock();
     if(!free_list) {
         int i;
         free_list=libspectrum_malloc(FREE_LIST_ALLOCATE_CHUNK*sizeof(GSList));
@@ -53,6 +68,7 @@ void    allocate_free   ( void ) {
             free_list[i].next=&free_list[i+1];
         free_list[FREE_LIST_ALLOCATE_CHUNK-1].next=NULL;
     }
+    unlock();
 }
 
 
@@ -71,8 +87,10 @@ GSList* g_slist_insert	(GSList		*list,
 
   allocate_free();
 
+  lock();
   new_list = free_list;
   free_list=free_list->next;
+  unlock();
   new_list->data = data;
   new_list->next=NULL;
 
@@ -119,8 +137,10 @@ GSList* g_slist_insert_sorted	(GSList		*list,
 
   if (!list)
     {
+      lock();
       new_list = free_list;
       free_list=free_list->next;
+      unlock();
       new_list->data = data;
       new_list->next=NULL;
       return new_list;
@@ -135,8 +155,10 @@ GSList* g_slist_insert_sorted	(GSList		*list,
       cmp = (*func) (data, tmp_list->data);
     }
 
+  lock();
   new_list = free_list;
   free_list=free_list->next;
+  unlock();
   new_list->data = data;
 
   if ((!tmp_list->next) && (cmp > 0))
@@ -292,8 +314,10 @@ void	g_slist_free		(GSList		*list) {
       while( last_node->next )
 	last_node = last_node->next;
 
+      lock();
       last_node->next = free_list;
       free_list = list;
+      unlock();
     }
 }
 
@@ -355,9 +379,11 @@ gint	g_slist_position	(GSList		*list,
 void
 libspectrum_slist_cleanup( void )
 {
+  lock();
   libspectrum_free( allocated_list );
   allocated_list = NULL;
   free_list = NULL;
+  unlock();
 }
 
 #endif				/* #ifndef HAVE_LIB_GLIB */
