@@ -651,9 +651,9 @@ pzx_read_string( const libspectrum_byte **ptr, const libspectrum_byte *end,
     *(buffer + length++) = **ptr; (*ptr)++;
   }
 
-  /* Advance past the null terminator if it isn't the end of the block */
-  if( **ptr == '\0' && *ptr < end ) (*ptr)++;
-  
+  /* Advance past the null terminator discarding any garbage */
+  *ptr = end;
+
   *dest = libspectrum_new( char, (length + 1) );
 
   strncpy( *dest, buffer, length );
