@@ -206,6 +206,16 @@ libspectrum_snap_alloc( void )
     libspectrum_snap_set_divide_ram( snap, i, NULL );
   }
 
+  libspectrum_snap_set_divmmc_active( snap, 0 );
+  libspectrum_snap_set_divmmc_eprom_writeprotect( snap, 0 );
+  libspectrum_snap_set_divmmc_paged( snap, 0 );
+  libspectrum_snap_set_divmmc_control( snap, 0 );
+  libspectrum_snap_set_divmmc_pages( snap, 0 );
+  libspectrum_snap_set_divmmc_eprom( snap, 0, NULL );
+  for( i = 0; i < SNAPSHOT_DIVMMC_PAGES; i++ ) {
+    libspectrum_snap_set_divmmc_ram( snap, i, NULL );
+  }
+
   libspectrum_snap_set_fuller_box_active( snap, 0 );
 
   libspectrum_snap_set_melodik_active( snap, 0 );
@@ -259,6 +269,12 @@ libspectrum_snap_alloc( void )
   libspectrum_snap_set_covox_active( snap, 0 );
   libspectrum_snap_set_covox_dac( snap, 0 );
 
+  libspectrum_snap_set_ulaplus_active( snap, 0 );
+  libspectrum_snap_set_ulaplus_palette_enabled( snap, 0 );
+  libspectrum_snap_set_ulaplus_current_register( snap, 0 );
+  libspectrum_snap_set_ulaplus_palette( snap, 0, NULL );
+  libspectrum_snap_set_ulaplus_ff_register( snap, 0 );
+
   libspectrum_snap_set_multiface_active( snap, 0 );
   libspectrum_snap_set_multiface_paged( snap, 0 );
   libspectrum_snap_set_multiface_model_one( snap, 0 );
@@ -270,6 +286,8 @@ libspectrum_snap_alloc( void )
   libspectrum_snap_set_multiface_ram( snap, 0, NULL );
   libspectrum_snap_set_multiface_ram_length( snap, 0, 0 );
 
+  libspectrum_snap_set_zxmmc_active( snap, 0 );
+
   return snap;
 }
 
@@ -331,6 +349,17 @@ libspectrum_snap_free( libspectrum_snap *snap )
   if( libspectrum_snap_didaktik80_ram( snap, 0 ) )
     libspectrum_free( libspectrum_snap_didaktik80_ram( snap, 0 ) );
 
+  libspectrum_free( libspectrum_snap_divide_eprom( snap, 0 ) );
+  for( i = 0; i < SNAPSHOT_DIVIDE_PAGES; i++ )
+    libspectrum_free( libspectrum_snap_divide_ram( snap, i ) );
+
+  libspectrum_free( libspectrum_snap_divmmc_eprom( snap, 0 ) );
+  for( i = 0; i < SNAPSHOT_DIVMMC_PAGES; i++ )
+    libspectrum_free( libspectrum_snap_divmmc_ram( snap, i ) );
+
+  if( libspectrum_snap_ulaplus_palette( snap, 0 ) )
+    libspectrum_free( libspectrum_snap_ulaplus_palette( snap, 0 ) );
+
   if( libspectrum_snap_multiface_ram( snap, 0 ) )
     libspectrum_free( libspectrum_snap_multiface_ram( snap, 0 ) );
 
